// scripts.js

// Globals

var refresh = 0;		// holds the Milliseconds of the refresh-interval
var buddyname = "";		// name of the buddy
var buddyproto = "";
var realname = null;		// real name of the buddy
var showStatusMessage = false;
var defaultChatApp = "iChat";

var buddyIsOnline = false;
var statusMessage = "";			// Status Message from iChat
var lastseen = 0;			// timestamp when buddy was online the last time
var idleTime = 0;			// minutes of time idle

var interval = null;
var serverresponse = null;		// response of oscar.aol.com
var req = false;

function showVersion() {
	document.getElementById('infospan').innerHTML = "v 0.3";
}


// Dashboard Setup on first load
function setup() {
	if(window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		widget.onremove = doCleanup;
	}
	
	exitflip();
	buddyname = getBuddyName();
	buddyproto = getBuddyProto();
	realname = getRealName();
	lastseen = getLastseen();
	refresh = getRefreshTime();
	showStatusMessage = getShowStatusMessage();
	defaultChatApp = getDefaultChatApp();

	createGenericButton(document.getElementById("done"), "done", savePrefs);	
	addProtoPossibilities();
	addChatAppPossibilities();
	addRefreshPossibilities();
	
	checkState();
	
	if(interval) {
		clearInterval(interval);
		interval = null;
	}
	
	if(refresh > 1000)
		interval = setInterval("checkState()", refresh);
}




/*
 * Main functions
 */


function checkState() {
	setState("checking");
	document.getElementById('flip').style.display = 'none';
	var processing;
	
	if(!buddyname)
		buddyname = getBuddyName();
	
	if(buddyname) {
		// iChat is running
		if(iChatRunning() && iChatLoggedIn())
			setStatusFromApp("iChat");
		
		// Adium is running
		else if(adiumRunning() && adiumLoggedIn())
			setStatusFromApp("Adium");
		
		// iChat is not running
		else {
			statusMessage = "";
			var requestURL = (buddyproto == "jabber") ?
					"http://edgar.netflint.net/status.php?jid=" + buddyname + "&type=text" :
					"http://big.oscar.aol.com/" + buddyname + "?on_url=http://www.google.com/nav_page.gif&off_url=http://www.google.com/nav_current.gif";
			HTTPRequest(requestURL);
		}
	}
	else
		setState("nobuddy");
}


function HTTPRequest(url) {

	// test for native XMLHttpRequest object
	if(window.XMLHttpRequest) {
		try {
			req = new XMLHttpRequest();
		} catch(e) {
			req = false;
		}
	}
	
	// process
	if(req) {
		req.onreadystatechange = (buddyproto == "jabber") ? jabberReqChange : AIMReqChange;
		req.open("GET", url, true);
		req.setRequestHeader("Cache-Control", "no-cache");
		
		req.send(null);
		return 1;
	}
	return 0;
}

function AIMReqChange() {
	
	// only if req shows "loaded" ( == state 4) and status is "OK" (200)
	if(req.readyState == 4) {
		if((req.status >= 200) && (req.status < 400)) {
			serverresponse = encodeURI(req.responseText.substring(10, 20));
			buddyIsOnline = (serverresponse == "%C3%BD%C3%BB%C3%95%C3%91%C3%81%C3%9A%C2%AB%03%C2%A2%C2%93");
			setState(buddyIsOnline ? "online" : "offline");
			if(buddyIsOnline) {
				lastseen = new Date().getTime();
				saveLastseen();
			}
		}
		else {
			serverresponse = null;
			buddyIsOnline = false;
			setState("noresponse");
		}
	}
}

function jabberReqChange() {
	
	// only if req shows "loaded" ( == state 4) and status is "OK" (200)
	if(req.readyState == 4) {
		if((req.status >= 200) && (req.status < 400)) {
			serverresponse = req.responseText.split(": ");
			buddyIsOnline = ((serverresponse[0] != "Not Available") && (serverresponse[0] != "Offline"));
			statusMessage = serverresponse[1] ? serverresponse[1] : "";
			//alert("status: '" + statusMessage + "' and response: '" + serverresponse[0] + "'");
			
			switch(serverresponse[0]) {
				case "Online":
				case "Free for chat":
					setState("online");
					break;
				
				case "Away":
				case "Do not disturb":
					setState("away");
					break;
				
				case "Not Available":
				case "Offline":
					setState("offline");
					break;
					
				default:
					setState("unknown");
					break;
			}
						
			if(buddyIsOnline) {
				lastseen = new Date().getTime();
				saveLastseen();
			}
		}
		else {
			serverresponse = null;
			buddyIsOnline = false;
			setState("noresponse");
		}
	}
}


function setStatusFromApp(app) {
	if(!buddyname)
		setState("nobuddy");
	//alert("Using App " + app);
	if(app != "iChat" && app != "Adium") {
		setState("error");
		return;
	}
	
	var status;
	var ASstring;
	
	if(widget.system && ((app == "iChat") ? iChatRunning() : adiumRunning())) {
		ASstring = (app == "iChat") ?
				"tell application \"iChat\" to return the status of the first account whose id is \"" + (buddyproto == "jabber" ? "Jabber:" : "AIM:") + buddyname + "\"" :
				"tell application \"Adium\" to return the status of the first contact whose UID is \"" + buddyname + "\"";
		status = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
		alert(ASstring + " " + status);
		status = status ? status.replace(/\n/g, "") : "offline";
	}
	
	if(status == "available") {
		if(showStatusMessage) {
			ASstring = (app == "iChat") ?
				"tell application \"iChat\" to return the status message of the first account whose id is \"" + (buddyproto == "jabber" ? "Jabber:" : "AIM:") + buddyname + "\"" :
				"tell application \"Adium\" to return the status message of the first contact whose UID is \"" + buddyname + "\"";
			statusMessage = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
			statusMessage = statusMessage ? statusMessage.replace(/\n/g, "") : "";
		}
		setState("online");
		lastseen = new Date().getTime();
		saveLastseen();
		buddyIsOnline = true;
	}
	else if(status == "away") {
		if(showStatusMessage) {
			ASstring = (app == "iChat") ?
				"tell application \"iChat\" to return the status message of the first account whose id is \"" + (buddyproto == "jabber" ? "Jabber:" : "AIM:") + buddyname + "\"" :
				"tell application \"Adium\" to return the status message of the first contact whose UID is \"" + buddyname + "\"";
			statusMessage = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
			statusMessage = statusMessage ? statusMessage.replace(/\n/g, "") : "";
		}
		setState("away");
		lastseen = new Date().getTime();
		saveLastseen();
		buddyIsOnline = true;
	}
	else if(status == "idle") {
		ASstring = (app == "iChat") ?
			"tell application \"iChat\" to return the idle time of the first account whose id is \"" + (buddyproto == "jabber" ? "Jabber:" : "AIM:") + buddyname + "\"" :
			"tell application \"Adium\" to return the idle time of the first contact whose UID is \"" + buddyname + "\"";
		idleTime = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
		idleTime = idleTime ? idleTime.replace(/\D/g, "") : 0;
		setState("idle");
		lastseen = new Date().getTime();
		saveLastseen();
		buddyIsOnline = true;
	}
	else if(status == "offline") {
		setState("offline");
		buddyIsOnline = false;
	}
	else if(status == "unknown") {
		setState("error");
	}
	else {
		alert("App " + app + " returned status: '" + status + "'");
		setState("error");
	}
}



function getState() {
	return buddyIsOnline;
}


function setState(state) {
	var obj = document.getElementById("the_buddy");
	var bg_left = document.getElementById("buddy_left").style;
	var bg_middle = document.getElementById("buddy_middle").style;
	var bg_right = document.getElementById("buddy_right").style;
	var last = document.getElementById('lastseen');
	
	document.getElementById('flip').style.display = 'block';
	mouseexit();
	
	switch(state) {
		case "checking":
			bg_left.backgroundImage = "url(Images/orange_left.png)";
			bg_middle.backgroundImage = "url(Images/orange_middle.png)";
			bg_right.backgroundImage = "url(Images/orange_right.png)";
			obj.innerHTML = "checking...";
			adjustSize(140);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", null);
			last.innerHTML = showStatusMessage ? realname : "";
			break;
		
		case "online":
			bg_left.backgroundImage = "url(Images/green_left.png)";
			bg_middle.backgroundImage = "url(Images/green_middle.png)";
			bg_right.backgroundImage = "url(Images/green_right.png)";
			obj.innerHTML = realname;
			adjustSize();
			obj.setAttribute("class", "online");
			obj.setAttribute("onclick", "openChat('" + buddyname + "');");
			last.innerHTML = showStatusMessage ? statusMessage : "";
			break;
		
		case "offline":
			bg_left.backgroundImage = "url(Images/magenta_left.png)";
			bg_middle.backgroundImage = "url(Images/magenta_middle.png)";
			bg_right.backgroundImage = "url(Images/magenta_right.png)";
			obj.innerHTML = realname;
			adjustSize();
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			setLastseen();
			break;
		
		case "away":
			bg_left.backgroundImage = "url(Images/red_left.png)";
			bg_middle.backgroundImage = "url(Images/red_middle.png)";
			bg_right.backgroundImage = "url(Images/red_right.png)";
			obj.innerHTML = realname;
			adjustSize();
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = showStatusMessage ? statusMessage : "";
			break;
		
		
		case "idle":
			bg_left.backgroundImage = "url(Images/orange_left.png)";
			bg_middle.backgroundImage = "url(Images/orange_middle.png)";
			bg_right.backgroundImage = "url(Images/orange_right.png)";
			obj.innerHTML = realname;
			adjustSize();
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = showStatusMessage ? ("idle: " + Math.round(idleTime/60) + " mins") : "";
			break;
		
		case "nobuddy":
			bg_left.backgroundImage = "url(Images/gray_left.png)";
			bg_middle.backgroundImage = "url(Images/gray_middle.png)";
			bg_right.backgroundImage = "url(Images/gray_right.png)";
			obj.innerHTML = "no buddy";
			adjustSize(140);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "showPrefs();");
			last.innerHTML = "";
			break;
		
		case "unknown":
			bg_left.backgroundImage = "url(Images/gray_left.png)";
			bg_middle.backgroundImage = "url(Images/gray_middle.png)";
			bg_right.backgroundImage = "url(Images/gray_right.png)";
			obj.innerHTML = "unknown";
			adjustSize(140);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = realname;
			break;
		
		case "noresponse":
			bg_left.backgroundImage = "url(Images/black_left.png)";
			bg_middle.backgroundImage = "url(Images/black_middle.png)";
			bg_right.backgroundImage = "url(Images/black_right.png)";
			obj.innerHTML = "no response";
			adjustSize(162);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = showStatusMessage ? realname : "";
			break;
		
		case "error":
			bg_left.backgroundImage = "url(Images/black_left.png)";
			bg_middle.backgroundImage = "url(Images/black_middle.png)";
			bg_right.backgroundImage = "url(Images/black_right.png)";
			obj.innerHTML = "error";
			adjustSize(120);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = showStatusMessage ? realname : "";
			break;
			
		default:
			bg_left.backgroundImage = "url(Images/gray_left.png)";
			bg_middle.backgroundImage = "url(Images/gray_middle.png)";
			bg_right.backgroundImage = "url(Images/gray_right.png)";
			obj.innerHTML = "state?";
			adjustSize(140);
			obj.setAttribute("class", "offline");
			obj.setAttribute("onclick", "checkState();");
			last.innerHTML = "unknown state";
			break;
	}
}

function setLastseen(force) {		// formats and sets the time of last time the widget saw the buddy (if showStatusMessage is true)
	var stamp = force ? force : (lastseen ? lastseen : "");
	var obj = document.getElementById("lastseen");
	if(!showStatusMessage || !stamp) {
		obj.innerHTML = "";
		return;
	}
	
	
	var size = document.getElementById("front").style.width.replace(/[^\d-\.]+/, "");
	
	var date = new Date();
	date.setTime(stamp);
	var month = date.getMonth() + 1;
	var year = (date.getYear() + 1900).toString().substring(2, 4);
	var hour = (date.getHours() > 9) ? date.getHours() : "0" + date.getHours();
	var minutes = (date.getMinutes() > 9) ? date.getMinutes() : "0" + date.getMinutes();
	var display_this = (size > 88) ? (date.getDate() + "." + month + "." + year + " " + hour + ":" + minutes) : (date.getDate() + "." + month + "." + year);
	obj.innerHTML = display_this;
}

function openChat(buddy) {
	if(window.widget) {
		//var openAS = "tell application \"iChat\"\n\tactivate\n\tdelay 3\n\tsend \"hello\" to (accounts where handle is \"" + buddy + "\")\nend tell";
		//var AS = widget.system("/usr/bin/osascript -e '" + openAS + "'" , null).status;
		var AS = widget.openApplication((defaultChatApp == "iChat") ? "com.apple.iChat" : "com.adiumX.adiumX");
		return !AS;
	}
	return 0;
}









/*
 * Pref functions
 */

function adjustPrefs() {
	var form = document.forms[0];
	
	// Buddy name
	buddyname = buddyname ? buddyname : getBuddyName();
	form.elements['buddyname'].value = buddyname;
	
	// Buddy proto
	//addProtoPossibilities(); > done in "setup()"
	
	// Real name
	realname = realname ? realname : getRealName();
	form.elements['realname'].value = realname;
	
	// Timer
	//addRefreshPossibilities(); > done in "setup()"
	
	// show status message?
	showStatusMessage = showStatusMessage ? showStatusMessage : getShowStatusMessage();
	form.elements['showStatusMessage'].checked = showStatusMessage;
	
	// Chat App
	//addChatAppPossibilities(); > done in "setup()"
}

function getBuddyName() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('buddyname'));
	
	return value ? value : "";
}

function getBuddyProto() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('buddyproto'));
	
	return value ? value : "";
}

function getRealName() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('realname'));
	
	return value ? value : (getBuddyName() ? getBuddyName() : "");
}

function getLastseen() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('lastseen'));
	
	return value ? value : 0;
}

function getRefreshTime() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('refresh'));
	
	return value ? value * 1000 : 0;
}

function getShowStatusMessage() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('showStatusMessage'));
	
	return value ? true : false;
}

function getDefaultChatApp() {
	if(window.widget)
		var value = widget.preferenceForKey(prefKey('defaultChatApp'));
	
	return value ? value : "iChat";
}

function addRefreshPossibilities() {
	//alert("addRefreshPossibilities() called");
	
	var selector = document.forms[0].selector;
	var possarr = new Array();
	possarr[5] = "5 seconds";
	possarr[10] = "10 seconds";
	possarr[20] = "20 seconds";
	possarr[60] = "minute";
	possarr[300] = "5 minutes";
	possarr[600] = "10 minutes";
	possarr[1800] = "half hour";
	possarr[3600] = "hour";
	
	selector.appendChild(document.createElement("option"));
	selector.lastChild.value = "0";
	selector.lastChild.innerHTML = "refresh manually (click the widget to refresh)";
	selector.appendChild(document.createElement("option"));
	selector.lastChild.value = "";
	selector.lastChild.innerHTML = "---";
	
	for(var seconds in possarr) {
		selector.appendChild(document.createElement("option"));
		selector.lastChild.value = seconds;
		selector.lastChild.innerHTML = "Check every " + possarr[seconds];
		selector.lastChild.selected = (refresh == seconds * 1000);		// select the right one
	}
}

function addProtoPossibilities() {
	//alert("addProtoPossibilities() called");
	
	var selector = document.forms[0].proto;
	var possarr = new Array();
	possarr[0] = "AIM";
	possarr[1] = ".mac";
	possarr[2] = "jabber";
	
	for(var key in possarr) {
		selector.appendChild(document.createElement("option"));
		selector.lastChild.value = possarr[key];
		selector.lastChild.innerHTML = possarr[key];
		selector.lastChild.selected = (buddyproto == possarr[key]);		// select the right one
	}
}

function addChatAppPossibilities() {
	//alert("addChatAppPossibilities() called");
	
	var selector = document.forms[0].chatapp;
	var possarr = new Array();
	possarr[0] = "iChat";
	possarr[1] = "Adium";
	
	for(var key in possarr) {
		selector.appendChild(document.createElement("option"));
		selector.lastChild.value = possarr[key];
		selector.lastChild.innerHTML = possarr[key];
		selector.lastChild.selected = (defaultChatApp == possarr[key]);		// select the right one
	}
}

/*
function removePossibilities(name) {
	var selector = document.forms[0].name;
	if(!selector)
		return;
	
	var startlen = selector.childNodes.length;
	for(i = 0; i < startlen; i++) {
		selector.removeChild(selector.options[0]);
	}
}
*/

function prefKey(key) {
	if(window.widget)
		return widget.identifier + "_" + key;
	else
		return key;
}








/*
 * Turnaround, Size and Save Prefs
 */

function adjustSize(force) {
	var optsize;
	
	if(realname.length > 0)
		optsize = (realname.length * 11) + 34;
	else
		optsize = 140;
	
	optsize = force ? force : optsize;
	optsize = (optsize < 78) ? 78 : optsize;
	document.getElementById("front").style.width = optsize + "px";
	window.resizeTo(optsize, 50);
}

function showPrefs() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	adjustPrefs();		// creates the option-menu to select the refresh-time
	showVersion();		// sets version-number
	window.resizeTo(270, 70);	// sets window size
	
	if(window.widget)
		widget.prepareForTransition("ToBack");
	
	if(interval) {
		clearInterval(interval);
		interval = null;
	}
				
	front.style.display = "none";
	back.style.display = "block";
		
	if(window.widget)
		setTimeout("widget.performTransition();", 0);  
}

function saveLastseen() {
	var value = lastseen ? lastseen : null;
	
	if(window.widget)
		widget.setPreferenceForKey(value, prefKey('lastseen'));
}

function savePrefs() {
	var form = document.forms[0];
	
	var buddy = form.elements['buddyname'].value ? form.elements['buddyname'].value.replace(/\s+/, "") : "";
	var proto = form.proto.options[form.proto.selectedIndex].value;
	var real = form.elements['realname'].value ? form.elements['realname'].value : buddy;
	var showmsg = form.elements['showStatusMessage'].checked;
	var chapp = form.chatapp.options[form.chatapp.selectedIndex].value;
	var refr = form.selector.options[form.selector.selectedIndex].value ? form.selector.options[selector.selectedIndex].value : 0;
	
	if(proto == ".mac") {
		buddy = (buddy.indexOf("@mac.com") > 0) ? buddy : buddy + "@mac.com";
	}
	
	buddyname = buddy;
	buddyproto = proto;
	realname = real;
	showStatusMessage = showmsg;
	defaultChatApp = chapp;
	refresh = refr * 1000;
	
	if(window.widget) {
		widget.setPreferenceForKey(buddy, prefKey('buddyname'));
		widget.setPreferenceForKey(proto, prefKey('buddyproto'));
		widget.setPreferenceForKey(real, prefKey('realname'));
		widget.setPreferenceForKey(showmsg, prefKey('showStatusMessage'));
		widget.setPreferenceForKey(chapp, prefKey('defaultChatApp'));
		widget.setPreferenceForKey(refr, prefKey('refresh'));
		//alert("setting YAAOSC-prefs: buddyname=" + buddy + ":" + proto + ", realname=" + real + ", App=" + chapp + ", refresh=" + refr + ", showStatus=" + (showmsg ? "true" : "false"));
	}
	
	hidePrefs();
}


function hidePrefs() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if(window.widget)
		widget.prepareForTransition("ToFront");
	
	//document.getElementById("fliprollie").style.display = "none";	
	front.style.display = "block";
	back.style.display = "none";
	
	if(interval) {
		clearInterval(interval);
		interval = null;
	}
	
	if(refresh > 1000)
		interval = setInterval("checkState()", refresh);
		
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
	
	checkState();
}





/*
 * iChat and Adium
 */

function iChatRunning() {
	if(widget.system)
		return widget.system("ps -uwwx | grep \"iChat.app/Contents/MacOS/iChat\" | grep -v grep | wc -l", null).outputString.indexOf("1") == 7;
	else
		return 0;
}

function iChatLoggedIn() {
	if(widget.system) {
		var ASstring = "tell application \"iChat\" to return status";
		var ASstatus = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
		return (ASstatus.replace(/\n/, "") != "offline");
	}
	else
		return 0;
}

function adiumRunning() {
	if(widget.system)
		return widget.system("ps -uwwx | grep \"Adium.app/Contents/MacOS/Adium\" | grep -v grep | wc -l", null).outputString.indexOf("1") == 7;
	else
		return 0;
}

function adiumLoggedIn() {
	if(widget.system) {
		var ASstring = "tell application \"Adium\" to return my status";
		var ASstatus = widget.system("/usr/bin/osascript -e '" + ASstring + "'", null).outputString;
		return (ASstatus.replace(/\n/, "") != "offline");
	}
	else
		return 0;
}








/*
 * Handlers
 */


function onShow() {
	checkState();
	
	if(interval) {
		clearInterval(interval);
		interval = null;
	}
	
	if(refresh > 1000)
		interval = setInterval("checkState()", refresh);
}

function onHide() {
	if(interval)
		clearInterval(interval);
	
	interval = null;
}

function doCleanup() {
	if(window.widget) {
		widget.setPreferenceForKey(null, prefKey('buddyname'));
		widget.setPreferenceForKey(null, prefKey('realname'));
		widget.setPreferenceForKey(null, prefKey('refresh'));
		widget.setPreferenceForKey(null, prefKey('lastseen'));
	}
}





/*
 * i-Button by Apple
 */

var flipShown = false;
var animation = { duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null };

function mousemove(event) {
	if(!flipShown) {
		if (animation.timer != null) {
			clearInterval(animation.timer);
			animation.timer = null;
		}
 
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById('flip');
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}


function mouseexit(event) {
	if(flipShown) {
		
		// fade out the info button
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer	= null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
	
	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}

function limit_3(a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
	return from + (to - from) * ease;
}


// these functions are called when the info button itself receives onmouseover and onmouseout events (by Apple)

function enterflip(event) {
	document.getElementById("fliprollie").style.display = "block";
}

function exitflip(event) {
	document.getElementById("fliprollie").style.display = "none";
}

